from typing import ClassVar, Dict, Any

from celery import Task
from jsonschema import validate

CODE_INCOMING_DATA_ERROR = 'IncomingDataError'

TASK_SCHEMA = {
    "type": "object",
    "properties": {
        "is_succeed": {"type": "boolean"},
        "msg": {"type": "string"},
        "msg_code": {"type": "string"},
        "data": {},
    },
}


def check_schema(ins: Any) -> None:
    validate(instance=ins, schema=TASK_SCHEMA)


class FlaskSqlalchemyContext:
    from flask import Flask
    from flask_sqlalchemy import SQLAlchemy
    app_flask: ClassVar[Flask] = Flask(__name__)

    def __new__(cls, url: str, db: SQLAlchemy) -> None:
        cls.app_flask.config['SQLALCHEMY_DATABASE_URI'] = url
        db.init_app(cls.app_flask)
        return cls.app_flask.app_context()


class TaskBase(Task):
    from flask import Flask
    from flask_sqlalchemy import SQLAlchemy
    app_flask: ClassVar[Flask] = Flask(__name__)

    @classmethod
    def bind_db(cls, url: str, db: SQLAlchemy) -> None:
        """
        the method should be called early.
        :param url:
        :param db:
        :return:
        """
        cls.app_flask.config['SQLALCHEMY_DATABASE_URI'] = url
        db.init_app(cls.app_flask)

    @classmethod
    def task_result(cls, data: Dict[str, Any]) -> Dict[str, Any]:
        copied = data.copy()
        copied.setdefault('data', '')
        if copied.get('is_succeed'):
            copied.setdefault('msg', '')
            copied.setdefault('msg_code', '')
        validate(instance=copied, schema=TASK_SCHEMA)
        return data



from kombu import Exchange
from kombu import Queue

# REDIS_URL = 'redis://*******/10'
CELERY_TIMEZONE = 'Asia/Shanghai'
# BROKER_URL = REDIS_URL
# CELERY_RESULT_BACKEND = REDIS_URL
CELERY_RESULT_SERIALIZER = 'json'
CELERY_TASK_RESULT_EXTENDED = True
CELERY_RESULT_PERSISTENT = True
CELERY_TASK_SERIALIZER = 'json'
CELERY_TASK_RESULT_EXPIRES = None
CELERY_ACCEPT_CONTENT = ['json', 'json']
CELERY_INCLUDE = [
    'blueCup.task_inner_platform',
    'blueCup.task_notification',
    'blueCup.task_xinyan',
]

CELERY_ROUTES = {
    'blueCup.task_notification.*': {'queue': 'notification'},
}

CELERY_QUEUES = (
    Queue(name='notification', routing_key='task_notification.#',
          exchange=Exchange(name='Exchange_notification', type='topic')),
)

CELERY_TASK_DEFAULT_QUEUE='blueCup'


CELERY_TASK_ANNOTATIONS = {
    'send_email': {'rate_limit': '1/s'},
    'send_email_attach': {'rate_limit': '1/s'},
    'send_dingding_text': {'rate_limit': '1/s'},
    'send_dingding_markdown': {'rate_limit': '1/s'},
}
